@@ -147,10 +147,6 @@ FcConfigCreate (void)
     if (!config->configDirs)
 	goto bail1;
 
-    config->configMapDirs = FcStrSetCreate();
-    if (!config->configMapDirs)
-	goto bail1_5;
-
     config->configFiles = FcStrSetCreate ();
     if (!config->configFiles)
 	goto bail2;
@@ -230,8 +226,6 @@ bail4:
 bail3:
     FcStrSetDestroy (config->configFiles);
 bail2:
-    FcStrSetDestroy (config->configMapDirs);
-bail1_5:
     FcStrSetDestroy (config->configDirs);
 bail1:
     free (config);
@@ -362,40 +356,42 @@ FcConfigDestroy (FcConfig *config)
     FcExprPage	*page;
     FcMatchKind	k;
 
-    if (FcRefDec (&config->ref) != 1)
-	return;
-
-    (void) fc_atomic_ptr_cmpexch (&_fcConfig, config, NULL);
-
-    FcStrSetDestroy (config->configDirs);
-    FcStrSetDestroy (config->configMapDirs);
-    FcStrSetDestroy (config->fontDirs);
-    FcStrSetDestroy (config->cacheDirs);
-    FcStrSetDestroy (config->configFiles);
-    FcStrSetDestroy (config->acceptGlobs);
-    FcStrSetDestroy (config->rejectGlobs);
-    FcFontSetDestroy (config->acceptPatterns);
-    FcFontSetDestroy (config->rejectPatterns);
-
-    for (k = FcMatchKindBegin; k < FcMatchKindEnd; k++)
-	FcPtrListDestroy (config->subst[k]);
-    FcPtrListDestroy (config->rulesetList);
-    FcStrSetDestroy (config->availConfigFiles);
-    for (set = FcSetSystem; set <= FcSetApplication; set++)
-	if (config->fonts[set])
-	    FcFontSetDestroy (config->fonts[set]);
-
-    page = config->expr_pool;
-    while (page)
+    if (config)
     {
-      FcExprPage *next = page->next_page;
-      free (page);
-      page = next;
-    }
-    if (config->sysRoot)
+	if (FcRefDec (&config->ref) != 1)
+	    return;
+
+	(void) fc_atomic_ptr_cmpexch (&_fcConfig, config, NULL);
+
+	FcStrSetDestroy (config->configDirs);
+	FcStrSetDestroy (config->fontDirs);
+	FcStrSetDestroy (config->cacheDirs);
+	FcStrSetDestroy (config->configFiles);
+	FcStrSetDestroy (config->acceptGlobs);
+	FcStrSetDestroy (config->rejectGlobs);
+	FcFontSetDestroy (config->acceptPatterns);
+	FcFontSetDestroy (config->rejectPatterns);
+
+	for (k = FcMatchKindBegin; k < FcMatchKindEnd; k++)
+	    FcPtrListDestroy (config->subst[k]);
+	FcPtrListDestroy (config->rulesetList);
+	FcStrSetDestroy (config->availConfigFiles);
+	for (set = FcSetSystem; set <= FcSetApplication; set++)
+	    if (config->fonts[set])
+		FcFontSetDestroy (config->fonts[set]);
+
+	page = config->expr_pool;
+	while (page)
+	{
+	    FcExprPage *next = page->next_page;
+	    free (page);
+	    page = next;
+	}
+	if (config->sysRoot)
 	FcStrFree (config->sysRoot);
 
-    free (config);
+	free (config);
+    }
 }
 
 /*
@@ -421,7 +417,7 @@ FcConfigAddCache (FcConfig *config, FcCache *cache,
     if (fs)
     {
 	int	nref = 0;
-	
+
 	for (i = 0; i < fs->nfont; i++)
 	{
 	    FcPattern	*font = FcFontSetFont (fs, i);
@@ -505,7 +501,7 @@ FcConfigAddDirList (FcConfig *config, FcSetName set, FcStrSet *dirSet)
     dirlist = FcStrListCreate (dirSet);
     if (!dirlist)
         return FcFalse;
-	
+
     while ((dir = FcStrListNext (dirlist)))
     {
 	if (FcDebug () & FC_DBG_FONTSET)
@@ -534,7 +530,7 @@ FcConfigBuildFonts (FcConfig *config)
     config = FcConfigReference (config);
     if (!config)
 	return FcFalse;
-	
+
     fonts = FcFontSetCreate ();
     if (!fonts)
     {
@@ -1331,7 +1327,7 @@ FcConfigEvaluate (FcPattern *p, FcPattern *p_pat, FcMatchKind kind, FcExpr *e)
 	v.u.b = FcConfigCompareValue (&vl, e->op, &vr);
 	FcValueDestroy (vl);
 	FcValueDestroy (vr);
-	break;	
+	break;
     case FcOpOr:
     case FcOpAnd:
     case FcOpPlus:
@@ -1347,7 +1343,7 @@ FcConfigEvaluate (FcPattern *p, FcPattern *p_pat, FcMatchKind kind, FcExpr *e)
 	    switch ((int) vle.type) {
 	    case FcTypeDouble:
 		switch ((int) op) {
-		case FcOpPlus:	
+		case FcOpPlus:
 		    v.type = FcTypeDouble;
 		    v.u.d = vle.u.d + vre.u.d;
 		    break;
@@ -1396,7 +1392,7 @@ FcConfigEvaluate (FcPattern *p, FcPattern *p_pat, FcMatchKind kind, FcExpr *e)
 		    str = FcStrPlus (vle.u.s, vre.u.s);
 		    v.u.s = FcStrdup (str);
 		    FcStrFree (str);
-			
+
 		    if (!v.u.s)
 			v.type = FcTypeVoid;
 		    break;
@@ -2279,8 +2275,8 @@ FcConfigSubstituteWithPat (FcConfig    *config,
 	printf ("FcConfigSubstitute done");
 	FcPatternPrint (p);
     }
-bail1:
     FamilyTableClear (&data);
+bail1:
     if (elt)
 	free (elt);
     if (value)
@@ -2829,7 +2825,7 @@ FcConfigAppFontAddFile (FcConfig    *config,
 	}
 	FcConfigSetFonts (config, set, FcSetApplication);
     }
-	
+
     if (!FcFileScanConfig (set, subdirs, file, config))
     {
 	FcStrSetDestroy (subdirs);
@@ -2920,8 +2916,13 @@ FcConfigGlobAdd (FcConfig	*config,
 		 FcBool		accept)
 {
     FcStrSet	*set = accept ? config->acceptGlobs : config->rejectGlobs;
+	FcChar8	*realglob = FcStrCopyFilename(glob);
+	if (!realglob)
+		return FcFalse;
 
-    return FcStrSetAdd (set, glob);
+    FcBool	 ret = FcStrSetAdd (set, realglob);
+    FcStrFree(realglob);
+    return ret;
 }
 
 static FcBool
